unit fmMainSrc;

interface

uses
  Windows, Classes, Graphics, Forms, extctrls, controls, Messages, SysUtils ;

type
  TfmBLifeMainForm = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDblClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClick(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    Timer: TTimer;
    timing : boolean;
    lastime, ActStripStart , botMargin, RHMargin : integer;
    StartTime : dword;
    currtickcount, MouseMovetime  : dword;
    MouseMoves : dword;
    fBackGroundColor : TColor;
    fRunning : Boolean;
    fFontColor : TColor;
    fLineColor : TColor;
    Writecanvas : tcanvas;
    PrevRect, tcrect : TRect;
    headcolor : tcolor;
    ldt : tdatetime;
    timestr : string;

    procedure PaintToCanvas(acanvas : tcanvas);
    procedure ScrSaveFormPaint(acanvas : tcanvas);
    procedure drawHline (var tp : integer; acanvas : tcanvas);
    procedure drawtitle(var tp : integer; acanvas : tcanvas);
    procedure drawActivity (var tp : integer; acanvas : tcanvas);
    procedure drawCensus (var tp : integer; acanvas : tcanvas);
    procedure drawLegend (var tp : integer; acanvas : tcanvas);
    procedure drawHistoryList(var tp : integer;  acanvas : tcanvas);
    procedure drawVLines (tp, bt : integer; acanvas : tcanvas);
    procedure DoTimerTick;
    procedure TimerInit;
    procedure ResetVariables;
    procedure FreeHistoryList;
    procedure MakeNewCreatures;
    procedure LoadSettings;
    Procedure MakeObjects;
    Procedure FreeObjects;
    procedure SizeChanged;
    procedure AutoRecreate;
    procedure DoFormPaint;
    procedure DoStart;
    procedure AdjustScrollRects;
    procedure Evolve;
    procedure Restart;
    procedure CloseIfOk;
    procedure CheckMouseMoves;
    procedure ResetMouseMoves;
    procedure Timer1Timer(Sender: TObject);

  protected
    procedure CreateParams(var Params:TCreateParams); override;
    Procedure SetBackGroundColor(ABackGroundColor : TColor);
    Procedure SetRunning(ARunning : Boolean);
    Procedure SetFontColor(AFontColor : TColor);
    procedure TimerTickChange(TickNo : integer);
    Property FontColor : TColor Read fFontColor Write SetFontColor;
    Property BackGroundColor : TColor Read fBackGroundColor Write SetBackGroundColor;
    Property Running : Boolean Read fRunning Write SetRunning;
    Property LineColor : TColor Read fLineColor Write fLineColor;

  public
    { Public declarations }

  end;

var
  fmBLifeMainForm: TfmBLifeMainForm;

implementation
uses
  dmodule,
  EvoStoreSrc,
  TEvoCreatureSrc,
  TEvoCreatureListSrc,
  cybmanip;

{$R *.DFM}

procedure TfmBLifeMainForm.FormCreate(Sender: TObject);

begin
    MakeObjects;
    LoadSettings;
    DoStart;
end;

procedure TfmBLifeMainForm.FormDestroy(Sender: TObject);
begin
    stopping := true;
    while timing do ;
    running := false;
    if saveroptions.Evolution then Evolve;
    FreeObjects;
end;

procedure TfmBLifeMainForm.DoStart;

begin
    sizechanged;
    if IsInSaverPreviewMode then begin
       {run in preview window}
       GetWindowRect(ParamHandle, PrevRect);
       width:=PrevRect.Right-PrevRect.Left;
       height:=PrevRect.Bottom-PrevRect.Top;
       Writecanvas := SpareCanvases[scPreviewBuffer].canvas;
    end
    else begin
      {run in fullscreen}
      Writecanvas := self.canvas;
      SpareCanvases[scPreviewBuffer].width := 0;
      SpareCanvases[scPreviewBuffer].height := 0;
    end;
    Running := true;
    visible := true;
end;

procedure TfmBLifeMainForm.LoadSettings;

begin
    randomize;
    dy := 32;
    dx := dy;
    dx2 := dx div 2;
    stdrect := rect(0,0, dx2, dx2);
    top := 0;
    left := 0;
    pi360 := trunc(2*pi*360);
    BaseRHpanelwidth := 170;
    histdeltax := 4;
    CensusSize := sizeof(TCreatureCensus);
    maxtick := 1000;
    StartTime := gettickcount;
    ldt := now;
    timestr := formatdatetime('HH:nn', ldt);
    restarts := 0;
    MouseMoves := 0;
    lastime := -1;
    headcolor := clred;
    stopping := false;
    ResetVariables;
    with SaverOptions do begin
      TimerTickChange(TimerTick);
      ShowLine := Showlegend or ShowCensus or ShowHistory or ShowActivity;
      self.BackGroundColor := SaverOptions.BackGroundColor;
      self.fontcolor := SaverOptions.Textcolor;
      self.linecolor := SaverOptions.linecolor;
    end;
    with SaverOptions do begin
      vertlastsaverpos  := 0;
      Horizlastsaverpos  := 0;
    end;
end;

Procedure TfmBLifeMainForm.MakeObjects;

Begin
    EvoCreatureList := TEvoCreatureList.create;
    BreadCreatureList := TList.create;
    HistoryList := TList.create;
    CensusSortList  := TList.create;
    ReincarnationList := TList.create;
    InitBMPArrays;
End;

Procedure TfmBLifeMainForm.FreeObjects;

Begin
   FreeBMPArrays;
   BreadCreatureList.clear;
   BreadCreatureList.free;
   EvoCreatureList.free;
   FreeHistoryList;
   HistoryList.free;
   CensusSortList.free;
   ClearReincarnationList;
   ReincarnationList.free;
   freemem(CreatureCensus, CensusSize);
End;

procedure TfmBLifeMainForm.SizeChanged;

begin
    if SaverOptions.ShowLine then  RHpanelwidth := BaseRHpanelwidth
                            else  RHpanelwidth := 0;
     if IsInSaverPreviewMode then begin
       varclientheight := screen.height div 2;
       varclientwidth := screen.width div 2;
     end
     else begin
       varclientheight := screen.height;
       varclientwidth := screen.width;
    end;
    botMargin := varclientheight;
    RHMargin := varclientwidth - RHpanelwidth;
    my := pred(botMargin div dy);
    pmy := pred(my);
    mx := pred(RHMargin div dx);
    pmx := pred(mx);
    HistPointCcount := (RHpanelwidth div histdeltax)-3;
    SetBmpArrayDims(varclientwidth, varclientheight, SpareCanvases);
    drect := rect(0,0,varclientwidth,varclientheight);
    tcrect := drect;
    srect := drect;
    drect2 := drect;
    srect2 := drect;
    HistMult := varclientheight div 8;
    rd1 := RHMargin+d1;
    rd2 := RHMargin+d2;
    rd3 := RHMargin+d3;
    rd4 := RHMargin+d4;
    rd5 := RHMargin+d5;
    rd6 := RHMargin+d6;
    rd7 := RHMargin+d7;
    rdh := RHMargin+dh;
    rd8 := RHMargin+d8;
end;

procedure TfmBLifeMainForm.FormResize(Sender: TObject);

begin
    SizeChanged;
end;

procedure TfmBLifeMainForm.ResetVariables;

begin
    CurrIteration := 0;
    deadcount := 0;
    breadcount := 0;
    eatenCount := 0;
    paintcount := 0;
    freebmp := false;
    FreeHistoryList;
    MouseMovetime := gettickcount;
end;

procedure TfmBLifeMainForm.MakeNewCreatures;
Var
   lc, mc : integer;
   lc2 : integer;

begin
   ClearReincarnationList;
   for lc := GSOgre to GSBlueBird do begin
      mc := CreatureStores[lc].Startingcount;
      for lc2 := 1 to mc do begin
          if IniDataLoadedOK then EvoCreatureList.MakeCreature( CreatureStores[lc])
                             else EvoCreatureList.MakeCreature( DefCreatureStores[lc]);
          if (EvoCreatureList.count > SaverOptions.MaximumNumberOfCreatures) then exit;
      end;
   end;
end;

procedure TfmBLifeMainForm.PaintToCanvas(acanvas : tcanvas);

begin
  EvoCreatureList.paint(acanvas);
  ActStripStart := 0;
  with SaverOptions do begin
     if showline then drawtitle( ActStripStart, acanvas);
     if ShowActivity then drawActivity (ActStripStart, acanvas);
     if ShowcENSUS then drawCensus (ActStripStart, acanvas);
     if ShowLegend then drawLegend (ActStripStart, acanvas);
     if ShowHistory then drawHistoryList(ActStripStart, acanvas);
  end;
end;

procedure TfmBLifeMainForm.AdjustScrollRects;
Var
   tc, lc, mc : integer;

begin
  with saveroptions do begin
      lc := (curriteration mod saverdelay);
      if (lc = 0) then begin
        if Vertscroll then begin
          if Vertscrolldirup then begin
              mc := (Vertlastsaverpos * Vertscrollstep);
              if (mc > varclientheight) then begin
                mc := 0;
                vertlastsaverpos := 0;
              end;
              tc := varclientheight - mc;
          end
          else begin
              tc := (vertlastsaverpos * Vertscrollstep);
              mc := varclientheight-tc;
              if (mc < 0) then begin
                mc := 0;
                tc := varclientheight;
                vertlastsaverpos := 0;
              end;
          end;
          inc( vertlastsaverpos);
          vdrect := rect(0,0,varclientwidth,tc);
          vsrect := rect(0,mc,varclientwidth,varclientheight);
          vdrect2 := rect(0,tc,varclientwidth,varclientheight);
          vsrect2 := rect(0,0,varclientwidth,mc);
        end;
        if Horizscroll then begin
          if Horizscrolldirleft then begin
              mc := (HoriZlastsaverpos * Horizscrollstep);
              if (mc > varclientwidth) then begin
                mc := 0;
                Horizlastsaverpos := 0;
              end;
              tc := varclientwidth - mc;
          end
          else begin
              tc := (HoriZlastsaverpos * Horizscrollstep);
              mc := varclientwidth - tc;
              if (mc < 0) then begin
                mc := varclientwidth;
                Horizlastsaverpos := 0;
                tc := varclientwidth;
              end;
          end;
          inc( Horizlastsaverpos);
          drect := rect(0,0,tc,varclientheight);
          srect := rect(mc,0,varclientwidth,varclientheight);
          drect2 := rect(tc,0,varclientwidth,varclientheight);
          srect2 := rect(0,0,mc,varclientheight);
        end;
      end;
  end;
end;

procedure TfmBLifeMainForm.ScrSaveFormPaint(acanvas : tcanvas);
Var
   Dcanvas, Scanvas : tcanvas;

begin
  AdjustScrollRects;
  Scanvas := SpareCanvases[scDblBuffer].Canvas;
  with saveroptions do begin
      if Running then begin
          if (Horizscroll and Vertscroll) then begin
                Dcanvas := SpareCanvases[scScrollBuffer].Canvas;
                Dcanvas.CopyRect(dRect, SCanvas, srect);
                Dcanvas.CopyRect(dRect2, Scanvas, srect2);
                acanvas.CopyRect(vdRect, DCanvas, vsrect);
                acanvas.CopyRect(vdRect2, DCanvas, vsrect2);
           end
           else begin
             if Horizscroll then begin
                acanvas.CopyRect(dRect, SCanvas, srect);
                acanvas.CopyRect(dRect2, SCanvas, srect2);
             end
             else if Vertscroll then begin
                acanvas.CopyRect(vdRect, SCanvas, vsrect);
                acanvas.CopyRect(vdRect2, SCanvas, vsrect2);
             end
             else acanvas.CopyRect(ClientRect, SCanvas, tcRect);
          end;
      end
      else begin
        acanvas.CopyRect(ClientRect, SCanvas, tcRect);
      end;
  end;
end;

procedure TfmBLifeMainForm.DoFormPaint;
Var
   Scanvas : tcanvas;

begin
     Scanvas := SpareCanvases[scDblBuffer].Canvas;
     Scanvas.FillRect(tcrect);
     PaintToCanvas(Scanvas);
     ScrSaveFormPaint(Writecanvas);
     if IsInSaverPreviewMode then begin
       canvas.CopyRect(Clientrect, Writecanvas, tcrect);
     end;
end;

procedure TfmBLifeMainForm.FormPaint(Sender: TObject);

begin
  if timing or stopping then exit;
  DoFormPaint;
end;

procedure TfmBLifeMainForm.TimerInit;

begin
  if saveroptions.showhistory then begin
    getmem(CreatureCensus, CensusSize);
    historylist.add(CreatureCensus);
  end;
  fillchar(CreatureCensus^, CensusSize, 0);
  fillchar(MaleCreatureCensus, CensusSize, 0);
  fillchar(FeMaleCreatureCensus, CensusSize, 0);
  deadcount := 0;
  breadcount := 0;
  eatenCount := 0;
  paintcount := 0;
  ResetMouseMoves;
end;

procedure TfmBLifeMainForm.Evolve;

Var
   ti1,ti2,lc, mc : integer;
   tb2, tb : boolean;
   astr : string;

   procedure adjfield( var field : integer; maxval : integer);

   begin
       if tb then field := lessint(field+1, maxval)
             else field := greatint(field-1, 1);
   end;

begin
   for lc := GSOgre to GSBlueBird do begin
       if (CreatureCensus^[lc] = 0) then with CreatureStores[lc] do begin
          mc := trunc(random(SuccefStartingcount));
          tb := (random >= 0.5);
          case mc of
          efInfluencedistance : adjfield( Influencedistance, maxInfluencedistance);
             efNumberchildren : adjfield( Numberchildren, maxNumberchildren);
                  efMinenergy : begin
                                   adjfield( Minenergy, maxMinenergy);
                                   Energy := greatint(energy, 2*Minenergy);
                                end;
                  efMaxenergy : adjfield( Maxenergy, maxMaxenergy);
                     efEnergy : begin
                                   adjfield( Energy, maxEnergy);
                                   Energy := greatint(energy, 2*Minenergy);
                                end;
                   efStepsize : begin
                                   tb2 := (Stepsize <= 0);
                                   adjfield( Stepsize, maxStepsize);
                                   if (tb2 and (Stepsize > 0)) then Stepsize := -1*Stepsize;
                                end;
                   efFoodlist : begin
                                   tb2 := ((Stepsize > 0) and (CensusSortList.count > 0));
                                   if tb2 then begin
                                     ti1 := integer(CensusSortList[0]);
                                     ti2 := longrec(ti1).lo;
                                     astr :=  CreatureStores[ti2].displayname;
                                     if (pos(astr, foodlist) = 0) then begin
                                        foodlist := foodlist+','+astr;
                                     end;
                                   end;
                                end;
          end;
       end;
   end;
end;

procedure TfmBLifeMainForm.Restart;

begin
   EvoCreatureList.clearall;
   if saveroptions.Evolution then Evolve;
   inc(restarts);
   resetvariables;
   makenewcreatures;
end;

procedure TfmBLifeMainForm.AutoRecreate;
Var
   lc : integer;

begin
 with saveroptions do begin
   if Autorecreate then begin
      lc := paintcount;
      if ((lc = 0) or (lc > SaverOptions.MaximumNumberOfCreatures)) then Restart
      else if (lc = LastCreatCount) then begin
         if (NoStableIterations = Autorecreatestableiterations) then Restart
                                                                else inc( NoStableIterations);
      end
      else begin
         LastCreatCount := lc;
         NoStableIterations := 0;
      end;
   end;
 end;
end;

procedure TfmBLifeMainForm.DoTimerTick;

begin
    AutoRecreate;
    inc(CurrIteration);
    TimerInit;
    EvoCreatureList.MoveAll;
    EvoCreatureList.Tick;
    DoFormPaint;
end;

procedure TfmBLifeMainForm.Timer1Timer(Sender: TObject);
begin
  if timing or stopping then exit;
  timing := true;
  try
        DoTimerTick;
  finally
    timing := false;
  end;
end;

procedure TfmBLifeMainForm.FreeHistoryList;
Var
   lc, mc : integer;

begin
   mc := pred(historylist.count);
   for lc := 0 to mc do freemem(historylist[lc], CensusSize);
   historylist.clear;
   if (CensusSize > 0) then begin
     getmem(CreatureCensus, CensusSize);
     fillchar(CreatureCensus^, CensusSize, 0);
     historylist.add(CreatureCensus);
   end;
end;

procedure TfmBLifeMainForm.TimerTickChange(TickNo : integer);
Var
   lc : integer;

begin
   lc := greatint(1, tickno);
   lc := lessint(maxtickspeed, lc);
   if (tickno <> lc) then begin
      tickno := lc;
      SaverOptions.timertick := tickno;
   end;
end;

procedure TfmBLifeMainForm.drawHistoryList(var tp : integer;  acanvas : tcanvas);

Var
   stp, cy, maxc, minc, PHbotMargin, xp, bc, sc, lc, mc : integer;
   aPCreatureCensus : PCreatureCensus;
   tb2, tb : boolean;

begin
  stp := tp;
  if (tp = 0) then drawHline ( tp, acanvas);
  mc := pred(historylist.count);
  if (mc >  HistPointCcount) then bc := (mc - HistPointCcount)
                             else bc := 1;
  PHbotMargin :=  botMargin ;
  maxc := 0;
  minc := 0;
  with acanvas do begin
     font.style := [fsBold];
     tp := tp+hgap;
     textout(Rd6, tp, 'History');
     font.style := [];
     tp := tp+m1;
     pen.width := 2;
     if (mc > -1) then begin
         for sc := GSOgre to GSBlueBird do begin
           if (CreatureCensus^[sc] > 0)  then begin
               xp := RHMargin;
               aPCreatureCensus := PCreatureCensus(historylist[bc-1]);
               cy := PHbotMargin-(aPCreatureCensus^[sc]*HistMult);
               maxc := greatint(maxc, aPCreatureCensus^[sc]);
               moveto(xp, cy);
               inc(xp, histdeltax);
               pen.color := CreatureStores[sc].HistColorVal;
               for lc := bc to mc do begin
                  aPCreatureCensus := PCreatureCensus(historylist[lc]);
                  cy := PHbotMargin-(aPCreatureCensus^[sc]*HistMult);
                  maxc := greatint(maxc, aPCreatureCensus^[sc]);
                  lineto(xp, cy);
                  inc(xp, histdeltax);
               end;
           end;
         end;
         if (bc > 1) then begin
            for lc := 0 to pred(bc) do begin
              freemem(historylist[lc], CensusSize);
              historylist.delete(lc);
            end;
         end;
     end;
     xp := RHMargin - dx2;
     tb := (maxc > 25);
     tb2 := (maxc > 50);
     for sc := minc to maxc do begin
        cy := PHbotMargin-((sc+1)*HistMult);
        if tb then begin
          if tb2 then begin
             if ((sc mod 5) = 0) then textout( xp, cy, inttostr(sc));
          end
          else if not( odd(sc)) then textout( xp, cy, inttostr(sc));
        end
        else textout( xp, cy, inttostr(sc));
     end;
     if (maxc > 0) then HistMult := trunc((PHbotMargin - tp )/ maxc)
                   else HistMult := 15;
     HistMult := greatint(1, HistMult);
     tp := PHbotMargin;
     drawHline ( tp, acanvas);
     drawVLines (stp, tp,acanvas);
   end;
end;

procedure TfmBLifeMainForm.drawtitle(var tp : integer; acanvas : tcanvas);
Var
   ofs,stp, tx2, tx3, tx1, tx  : integer;
   ofc : tcolor;

begin
  with acanvas do begin
           font.style := [fsBold];
           ofs := font.size;
           font.size := 14;
           ofc := font.color;
           font.color := headcolor;
           stp := tp;
           drawHline ( tp, acanvas);
           tp := tp+h0;
           tx := Rd6;
           textout(tx, tp, TitleString);
           tp := tp + textheight(TitleString);
           font.color := ofc;
           font.size := ofs;
           drawHline ( tp, acanvas);
           drawVLines (stp, tp,acanvas);
  end;
end;

procedure TfmBLifeMainForm.drawActivity (var tp : integer; acanvas : tcanvas);
Var
   stp, tx2, tx3, tx1, tx  : integer;
   acol : tcolor;

begin
  with acanvas do begin
           font.style := [fsBold];
           stp := tp;
           if (tp = 0) then drawHline ( tp, acanvas);
           tp := tp+h0;
           font.style := [];
           tx := Rdh;
           tx1 := rd8;
           textout(tx, tp, 'Run For');
           currtickcount := gettickcount;
           tx2 := (currtickcount - starttime);
           Runstring := TickDeltaToTimeStr(TX2);
           textout(tx1, tp, Runstring);
           IF ((TX2 div msecspermn) > lastime) then begin
               ldt := now;
               timestr := formatdatetime('HH:nn', ldt);
               inc(lastime);
           end;
           tx := Rdh;
           tx1 := tx+stl;
           tx2 := tx1+ctl+h0;
           tx3 := tx2+ctl+h0+h0;
           textout(tx3, tp, timestr);

           tx3 := tx2+stl;
           tp := tp+m1;
           textout(tx, tp, 'Iteration');
           textout(tx1, tp, inttostr(CurrIteration));
           textout(tx2, tp, 'Resets');
           textout(tx3, tp, inttostr(Restarts));

           tp := tp+m1;
           textout(tx, tp, 'Total');
           textout(tx1, tp, inttostr(paintcount));
           if didnotbreed then begin
               acol := font.color;
               font.color := headcolor;
               textout(tx2, tp, 'Overflow');
               font.color := acol;
           end
           else begin
               textout(tx2, tp, 'Born');
               textout(tx3, tp, inttostr(breadcount));
           end;
           tp := tp+m1;
           textout(tx, tp, 'Died');
           textout(tx1, tp, inttostr(deadcount));
           textout(tx2, tp, 'Eaten');
           textout(tx3, tp, inttostr(eatenCount));
           tp := tp+m1;
           drawHline ( tp, acanvas);
           drawVLines (stp, tp,acanvas);
  end;
end;

function ClistSort(Item1, Item2: Pointer): Integer;

begin
   result := longrec(item2).hi - longrec(item1).hi;
   if (result = 0) then result := longrec(item2).lo - longrec(item1).lo;
end;

procedure TfmBLifeMainForm.drawCensus (var tp : integer; acanvas : tcanvas);
Var
   stp, lc, mc, tx1, tx  : integer;
   h1 : integer;

begin
  with acanvas do begin
     stp := tp;
     if (tp = 0) then drawHline ( tp, acanvas);
     h1 := tp+h0;
     font.style := [fsBold];
     textout(Rd6, h1, 'Census');
     MyDraw ( aCanvas, Rd3, h1, SexPics[esMale]);
     MyDraw ( aCanvas, Rd4, h1, SexPics[esfeMale]);
     font.style := [];
     CensusSortList.clear;
     for lc := GSOgre to GSBlueBird do begin
        longrec(tx).lo := lc;
        longrec(tx).hi := CreatureCensus^[lc];
        CensusSortList.add(pointer(tx));
     end;
     CensusSortList.Sort(ClistSort);
     for tx := GSOgre to GSBlueBird do begin
         tx1 := integer(CensusSortList[tx]);
         lc := longrec(tx1).lo;
         mc :=  m1*(tx+1)+h1;
         with CreatureStores[lc] do begin
           if (tx = GSOgre) then headcolor := HistColorVal;
           textout(Rd1, mc, Displayname);
           MyDraw ( aCanvas, Rd5, mc, GraphiC);
           if (CreatureCensus^[lc] > 0) then begin
             textout(Rd2, mc, inttostr(CreatureCensus^[lc]));
             textout(Rd3, mc, inttostr(MaleCreatureCensus[lc]));
             textout(Rd4, mc, inttostr(FemaleCreatureCensus[lc]));
             pen.width := 3;
             pen.color := HistColorVal;
             moveto(Rd6, mc+m2);
             lineto(Rd7, mc);
           end
           else textout(Rd4, mc, 'Extinct');
         end;
     end;
     tp := mc+m1;
     drawHline ( tp, acanvas);
     drawVLines (stp, tp,acanvas);
  end;
end;

procedure TfmBLifeMainForm.drawLegend (var tp : integer; acanvas : tcanvas);
Var
   stp, tx2,tx3, tx1, tx: integer;
   lc : tanimalstates;

begin
  with acanvas do begin
     stp := tp;
     if (tp = 0) then drawHline ( tp, acanvas);
     font.style := [fsBold];
     tp := tp+h0;
     textout(Rd6, tp, 'Legend');
     font.style := [];
     tx :=  RHMargin+ll;
     tx1 := tx+dx2+hgap;
     tx2 := tx1+tl+hgap;
     tx3 := tx2+dx2+hgap;
     tp := tp+m1;
     MyDraw ( aCanvas,tx , tp, SexPics[esfeMale]);
     textout(tx1, tp, 'Female');
     MyDraw ( aCanvas,tx2 , tp, SexPics[esMale]);
     textout(tx3, tp, 'Male');
     lc := taCompeting;
     while (lc >= taBreeding) do begin
          tp := tp+m1;
          MyDraw ( aCanvas,tx , tp, AnimalStatePics[lc]);
          textout(tx1, tp, AnimalStateStrings[lc]);
          if (lc = taIdle) then break;
          lc := pred(lc);
          MyDraw ( aCanvas,tx2 , tp, graphics.tbitmap(AnimalStatePics[lc]));
          textout(tx3, tp, AnimalStateStrings[lc]);
          lc := pred(lc);
     end;
     tp := tp+m1;
     drawHline ( tp, acanvas);
     drawVLines (stp, tp,acanvas);
  end;
end;

procedure TfmBLifeMainForm.drawHline (var tp : integer; acanvas : tcanvas);

begin
  with acanvas do begin
     pen.color := linecolor;
     moveto(RHMargin, tp);
     lineto(varclientwidth, tp);
  end;
end;

procedure TfmBLifeMainForm.drawVLines (tp, bt : integer; acanvas : tcanvas);
Var
   lc : integer;

begin
  with acanvas do begin
           pen.width := 3;
           pen.color := linecolor;
           moveto(RHMargin, tp);
           lineto(RHMargin, bt);
           lc := varclientwidth - hgap;
           moveto(lc, tp);
           lineto(lc, bt);
  end;
end;

procedure TfmBLifeMainForm.CreateParams(var Params:TCreateParams);
begin
  inherited CreateParams(Params);
  if (ConfigParent = 0) and (ParamHandle = 0) then exit;
  if (SaverMode = savModeExecute) then exit;
  with Params do begin
    if (ConfigParent <> 0)  then WndParent := ConfigParent;
    if (ParamHandle <> 0) then begin
      WndParent := ParamHandle;
      if (SaverMode = savModePreview) then Style := WS_CHILD or WS_CLIPSIBLINGS{ or WS_DISABLED or WS_VISIBLE{WS_CHILD };
    end;
  end;
end;

Procedure TfmBLifeMainForm.SetBackGroundColor(ABackGroundColor : TColor);
Var
   lc : integer;

Begin
     fBackGroundColor := ABackGroundColor;
     color := fBackGroundColor;
     canvas.brush.color := fBackGroundColor;
     CopyBmpArrayWithBackGr(fBackGroundColor, 17, AnimalStatePics, true);
     CopyBmpArrayWithBackGr(fBackGroundColor, isFemale, SexPics, true);
     SetBmpArrayBackGroundColor(fBackGroundColor, SpareCanvases);
     for lc := GSOgre to GSBlueBird do with CreatureStores[lc] do begin
         CopyStdBmpWithBackGr(fBackGroundColor, Bitmapindex, GraphiC);
     end;
End;

Procedure TfmBLifeMainForm.SetFontColor(AFontColor : TColor);
Begin
    fFontColor := AFontColor;
    canvas.font.color := fFontColor;
    SetBmpArrayFontColor(fFontColor, SpareCanvases);
End;

Procedure TfmBLifeMainForm.SetRunning(ARunning : Boolean);

Begin
  If (fRunning <> ARunning) then begin
    fRunning := ARunning;
    if fRunning then begin
           timer := ttimer.create(self);
           timer.enabled := false;
           with SaverOptions do begin
              if (timertick < 10) then timer.interval := maxtick div timertick
                                  else timer.interval := maxtick div (2*timertick);
           end;
           timer.ontimer := timer1timer;
           timer.enabled := true;
    end
    else begin
            timer.enabled := false;
            timer.ontimer := nil;
            timer.free;
    end;
  end;
End;

procedure TfmBLifeMainForm.FormDblClick(Sender: TObject);
begin
  closeifok;
end;

procedure TfmBLifeMainForm.CloseIfOk;

begin
  ReadRegistryForPasswordSetting;
  if (isWin9X and ScreenSaveUsePassword and (SaverMode = savModeExecute)) then begin
    if OKPassword then Close;
  end
  else Close;
end;

procedure TfmBLifeMainForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  closeifok;
end;

procedure TfmBLifeMainForm.FormClick(Sender: TObject);
begin
  closeifok;
end;

procedure TfmBLifeMainForm.CheckMouseMoves;
begin
    MouseMovetime  := gettickcount;
    MouseMoves := MouseMoves+1;
    if (MouseMoves > MouseMovesThreshold) then closeifok;
end;

procedure TfmBLifeMainForm.ResetMouseMoves;
Var
   lc, mc : integer;

begin
{    IF (MouseMovetime > currtickcount) THEN EXIT;}
    currtickcount := gettickcount;
    lc := (currtickcount - MouseMovetime);
    if (lc > MouseMoveResetTime) then MouseMoves := 0;
end;

procedure TfmBLifeMainForm.FormMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   CheckMouseMoves;
end;

procedure TfmBLifeMainForm.FormMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   CheckMouseMoves;
end;

{ End of code  }
end.


